@@ -41,12 +41,19 @@ gboolean	rb_uri_exists		(const char *uri);
 char *		rb_uri_resolve_relative	(const char *uri);
 gboolean	rb_uri_is_readable	(const char *uri);
 gboolean	rb_uri_is_writable	(const char *uri);
+gboolean	rb_uri_is_local		(const char *uri);
+char *		rb_uri_get_dir_name	(const char *uri);
 
 void		rb_uri_handle_recursively(const char *uri,
 					  GFunc func,
 					  gboolean *cancelflag,
 					  gpointer user_data);
 
+char*		rb_uri_append_path	(const char *uri,
+					 const char *path);
+char*		rb_uri_append_uri	(const char *uri,
+					 const char *fragment);
+
 void		rb_file_helpers_init	(void);
 void		rb_file_helpers_shutdown(void);
 
