@@ -146,11 +146,9 @@ rb_query_creator_constructor (GType type, guint n_construct_properties,
 {
 	RBQueryCreatorPrivate *priv;
 	RBQueryCreator *creator;
-	RBQueryCreatorClass *klass;
 	GladeXML *xml;
 	GtkWidget *mainbox;
 
-	klass = RB_QUERY_CREATOR_CLASS (g_type_class_peek (type));
 	creator = RB_QUERY_CREATOR (G_OBJECT_CLASS (rb_query_creator_parent_class)
 			->constructor (type, n_construct_properties, construct_properties));
 	priv = QUERY_CREATOR_GET_PRIVATE (creator);
@@ -362,7 +360,7 @@ rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->disjunction_check),
 					      disjunction);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->limit_check),
-					      limit_count || limit_size);
+					      limit_count || limit_size || limit_time);
 				
 		if (limit_count > 0) {
 			gtk_option_menu_set_history (GTK_OPTION_MENU (priv->limit_option), 0);
